@@ -24,11 +24,14 @@
 #include "base/exception.hpp"
 #include "base/tcpsocket.hpp"
 #include <boost/asio/buffer.hpp>
+#include <boost/asio/io_context_strand.hpp>
 #include <boost/asio/ip/tcp.hpp>
 #include <boost/asio/spawn.hpp>
 #include <boost/asio/ssl/context.hpp>
+#include <boost/date_time/posix_time/posix_time_duration.hpp>
 #include <boost/system/error_code.hpp>
 #include <climits>
+#include <cstdint>
 #include <fstream>
 #include <memory>
 #include <openssl/ssl.h>
@@ -435,7 +438,9 @@ void ApiListener::ListenerCoroutineProc(boost::asio::yield_context yc, const std
 
 			server->async_accept(sslConn->lowest_layer(), yc);
 
-			IoEngine::SpawnCoroutine(io, [this, sslConn](asio::yield_context yc) { NewClientHandler(yc, sslConn, String(), RoleServer); });
+			auto strand (Shared<asio::io_context::strand>::Make(io));
+
+			IoEngine::SpawnCoroutine(*strand, [this, strand, sslConn](asio::yield_context yc) { NewClientHandler(yc, strand, sslConn, String(), RoleServer); });
 		} catch (const std::exception& ex) {
 			Log(LogCritical, "ApiListener")
 				<< "Cannot accept new connection: " << ex.what();
@@ -461,8 +466,9 @@ void ApiListener::AddConnection(const Endpoint::Ptr& endpoint)
 	}
 
 	auto& io (IoEngine::Get().GetIoContext());
+	auto strand (Shared<asio::io_context::strand>::Make(io));
 
-	IoEngine::SpawnCoroutine(io, [this, endpoint, &io, sslContext](asio::yield_context yc) {
+	IoEngine::SpawnCoroutine(*strand, [this, strand, endpoint, &io, sslContext](asio::yield_context yc) {
 		String host = endpoint->GetHost();
 		String port = endpoint->GetPort();
 
@@ -474,7 +480,7 @@ void ApiListener::AddConnection(const Endpoint::Ptr& endpoint)
 
 			Connect(sslConn->lowest_layer(), host, port, yc);
 
-			NewClientHandler(yc, sslConn, endpoint->GetName(), RoleClient);
+			NewClientHandler(yc, strand, sslConn, endpoint->GetName(), RoleClient);
 
 			endpoint->SetConnecting(false);
 			Log(LogInformation, "ApiListener")
@@ -488,10 +494,13 @@ void ApiListener::AddConnection(const Endpoint::Ptr& endpoint)
 	});
 }
 
-void ApiListener::NewClientHandler(boost::asio::yield_context yc, const std::shared_ptr<AsioTlsStream>& client, const String& hostname, ConnectionRole role)
+void ApiListener::NewClientHandler(
+	boost::asio::yield_context yc, const Shared<boost::asio::io_context::strand>::Ptr& strand,
+	const std::shared_ptr<AsioTlsStream>&client, const String& hostname, ConnectionRole role
+)
 {
 	try {
-		NewClientHandlerInternal(yc, client, hostname, role);
+		NewClientHandlerInternal(yc, strand, client, hostname, role);
 	} catch (const std::exception& ex) {
 		Log(LogCritical, "ApiListener")
 			<< "Exception while handling new API client connection: " << DiagnosticInformation(ex, false);
@@ -506,7 +515,10 @@ void ApiListener::NewClientHandler(boost::asio::yield_context yc, const std::sha
  *
  * @param client The new client.
  */
-void ApiListener::NewClientHandlerInternal(boost::asio::yield_context yc, const std::shared_ptr<AsioTlsStream>& client, const String& hostname, ConnectionRole role)
+void ApiListener::NewClientHandlerInternal(
+	boost::asio::yield_context yc, const Shared<boost::asio::io_context::strand>::Ptr& strand,
+	const std::shared_ptr<AsioTlsStream>& client, const String& hostname, ConnectionRole role
+)
 {
 	namespace asio = boost::asio;
 	namespace ssl = asio::ssl;
@@ -533,7 +545,34 @@ void ApiListener::NewClientHandlerInternal(boost::asio::yield_context yc, const
 
 	boost::system::error_code ec;
 
-	sslConn.async_handshake(role == RoleClient ? sslConn.client : sslConn.server, yc[ec]);
+	{
+		struct DoneHandshake
+		{
+			bool Done = false;
+		};
+
+		auto doneHandshake (Shared<DoneHandshake>::Make());
+
+		IoEngine::SpawnCoroutine(*strand, [strand, client, doneHandshake](asio::yield_context yc) {
+			namespace sys = boost::system;
+
+			{
+				boost::asio::deadline_timer timer (strand->context());
+				timer.expires_from_now(boost::posix_time::microseconds(intmax_t(Configuration::TlsHandshakeTimeout * 1000000)));
+
+				sys::error_code ec;
+				timer.async_wait(yc[ec]);
+			}
+
+			if (!doneHandshake->Done) {
+				sys::error_code ec;
+				client->lowest_layer().cancel(ec);
+			}
+		});
+
+		sslConn.async_handshake(role == RoleClient ? sslConn.client : sslConn.server, yc[ec]);
+		doneHandshake->Done = true;
+	}
 
 	if (ec) {
 		// https://github.com/boostorg/beast/issues/915
@@ -657,19 +696,21 @@ void ApiListener::NewClientHandlerInternal(boost::asio::yield_context yc, const
 	if (ctype == ClientJsonRpc) {
 		Log(LogNotice, "ApiListener", "New JSON-RPC client");
 
+		if (endpoint && endpoint->GetConnected()) {
+			Log(LogNotice, "ApiListener")
+				<< "Ignoring JSON-RPC connection " << conninfo
+				<< ". We're already connected to Endpoint '" << endpoint->GetName() << "'.";
+			return;
+		}
+
 		JsonRpcConnection::Ptr aclient = new JsonRpcConnection(identity, verify_ok, client, role);
 
 		if (endpoint) {
-			bool needSync = !endpoint->GetConnected();
-
 			endpoint->AddClient(aclient);
 
-			IoEngine::SpawnCoroutine(IoEngine::Get().GetIoContext(), [this, aclient, endpoint, needSync](asio::yield_context yc) {
-				CpuBoundWork syncClient (yc);
-
-				SyncClient(aclient, endpoint, needSync);
+			Utility::QueueAsyncCallback([this, aclient, endpoint]() {
+				SyncClient(aclient, endpoint, true);
 			});
-
 		} else if (!AddAnonymousClient(aclient)) {
 			Log(LogNotice, "ApiListener")
 				<< "Ignoring anonymous JSON-RPC connection " << conninfo
@@ -1010,6 +1051,17 @@ void ApiListener::SyncSendMessage(const Endpoint::Ptr& endpoint, const Dictionar
 	}
 }
 
+/**
+ * Relay a message to a directly connected zone or to a global zone.
+ * If some other zone is passed as the target zone, it is not relayed.
+ *
+ * @param targetZone The zone to relay to
+ * @param origin Information about where this message is relayed from (if it was not generated locally)
+ * @param message The message to relay
+ * @param currentZoneMaster The current master node of the local zone
+ * @return true if the message has been relayed to all relevant endpoints,
+ *         false if it hasn't and must be persisted in the replay log
+ */
 bool ApiListener::RelayMessageOne(const Zone::Ptr& targetZone, const MessageOrigin::Ptr& origin, const Dictionary::Ptr& message, const Endpoint::Ptr& currentZoneMaster)
 {
 	ASSERT(targetZone);
@@ -1028,82 +1080,88 @@ bool ApiListener::RelayMessageOne(const Zone::Ptr& targetZone, const MessageOrig
 
 	std::vector<Endpoint::Ptr> skippedEndpoints;
 
-	bool relayed = false, log_needed = false, log_done = false;
-
-	std::set<Endpoint::Ptr> targetEndpoints;
-
+	std::set<Zone::Ptr> allTargetZones;
 	if (targetZone->GetGlobal()) {
-		targetEndpoints = localZone->GetEndpoints();
-
+		/* if the zone is global, the message has to be relayed to our local zone and direct children */
+		allTargetZones.insert(localZone);
 		for (const Zone::Ptr& zone : ConfigType::GetObjectsByType<Zone>()) {
-			/* Fetch immediate child zone members */
 			if (zone->GetParent() == localZone) {
-				std::set<Endpoint::Ptr> endpoints = zone->GetEndpoints();
-				targetEndpoints.insert(endpoints.begin(), endpoints.end());
+				allTargetZones.insert(zone);
 			}
 		}
 	} else {
-		targetEndpoints = targetZone->GetEndpoints();
+		/* whereas if it's not global, the message is just relayed to the zone itself */
+		allTargetZones.insert(targetZone);
 	}
 
-	for (const Endpoint::Ptr& targetEndpoint : targetEndpoints) {
-		/* Don't relay messages to ourselves. */
-		if (targetEndpoint == localEndpoint)
-			continue;
+	bool needsReplay = false;
 
-		log_needed = true;
+	for (const Zone::Ptr& currentTargetZone : allTargetZones) {
+		bool relayed = false, log_needed = false, log_done = false;
 
-		/* Don't relay messages to disconnected endpoints. */
-		if (!targetEndpoint->GetConnected()) {
-			if (targetZone == localZone)
-				log_done = false;
+		for (const Endpoint::Ptr& targetEndpoint : currentTargetZone->GetEndpoints()) {
+			/* Don't relay messages to ourselves. */
+			if (targetEndpoint == localEndpoint)
+				continue;
 
-			continue;
+			log_needed = true;
+
+			/* Don't relay messages to disconnected endpoints. */
+			if (!targetEndpoint->GetConnected()) {
+				if (currentTargetZone == localZone)
+					log_done = false;
+
+				continue;
+			}
+
+			log_done = true;
+
+			/* Don't relay the message to the zone through more than one endpoint unless this is our own zone.
+			 * 'relayed' is set to true on success below, enabling the checks in the second iteration.
+			 */
+			if (relayed && currentTargetZone != localZone) {
+				skippedEndpoints.push_back(targetEndpoint);
+				continue;
+			}
+
+			/* Don't relay messages back to the endpoint which we got the message from. */
+			if (origin && origin->FromClient && targetEndpoint == origin->FromClient->GetEndpoint()) {
+				skippedEndpoints.push_back(targetEndpoint);
+				continue;
+			}
+
+			/* Don't relay messages back to the zone which we got the message from. */
+			if (origin && origin->FromZone && currentTargetZone == origin->FromZone) {
+				skippedEndpoints.push_back(targetEndpoint);
+				continue;
+			}
+
+			/* Only relay message to the zone master if we're not currently the zone master.
+			 * e1 is zone master, e2 and e3 are zone members.
+			 *
+			 * Message is sent from e2 or e3:
+			 *   !isMaster == true
+			 *   targetEndpoint e1 is zone master -> send the message
+			 *   targetEndpoint e3 is not zone master -> skip it, avoid routing loops
+			 *
+			 * Message is sent from e1:
+			 *   !isMaster == false -> send the messages to e2 and e3 being the zone routing master.
+			 */
+			bool isMaster = (currentZoneMaster == localEndpoint);
+
+			if (!isMaster && targetEndpoint != currentZoneMaster) {
+				skippedEndpoints.push_back(targetEndpoint);
+				continue;
+			}
+
+			relayed = true;
+
+			SyncSendMessage(targetEndpoint, message);
 		}
 
-		log_done = true;
-
-		/* Don't relay the message to the zone through more than one endpoint unless this is our own zone.
-		 * 'relayed' is set to true on success below, enabling the checks in the second iteration.
-		 */
-		if (relayed && targetZone != localZone) {
-			skippedEndpoints.push_back(targetEndpoint);
-			continue;
+		if (log_needed && !log_done) {
+			needsReplay = true;
 		}
-
-		/* Don't relay messages back to the endpoint which we got the message from. */
-		if (origin && origin->FromClient && targetEndpoint == origin->FromClient->GetEndpoint()) {
-			skippedEndpoints.push_back(targetEndpoint);
-			continue;
-		}
-
-		/* Don't relay messages back to the zone which we got the message from. */
-		if (origin && origin->FromZone && targetZone == origin->FromZone) {
-			skippedEndpoints.push_back(targetEndpoint);
-			continue;
-		}
-
-		/* Only relay message to the zone master if we're not currently the zone master.
-		 * e1 is zone master, e2 and e3 are zone members.
-		 *
-		 * Message is sent from e2 or e3:
-		 *   !isMaster == true
-		 *   targetEndpoint e1 is zone master -> send the message
-		 *   targetEndpoint e3 is not zone master -> skip it, avoid routing loops
-		 *
-		 * Message is sent from e1:
-		 *   !isMaster == false -> send the messages to e2 and e3 being the zone routing master.
-		 */
-		bool isMaster = (currentZoneMaster == localEndpoint);
-
-		if (!isMaster && targetEndpoint != currentZoneMaster) {
-			skippedEndpoints.push_back(targetEndpoint);
-			continue;
-		}
-
-		relayed = true;
-
-		SyncSendMessage(targetEndpoint, message);
 	}
 
 	if (!skippedEndpoints.empty()) {
@@ -1113,7 +1171,7 @@ bool ApiListener::RelayMessageOne(const Zone::Ptr& targetZone, const MessageOrig
 			skippedEndpoint->SetLocalLogPosition(ts);
 	}
 
-	return !log_needed || log_done;
+	return !needsReplay;
 }
 
 void ApiListener::SyncRelayMessage(const MessageOrigin::Ptr& origin,
