@@ -15,53 +15,34 @@ using namespace icinga;
 
 REGISTER_APIFUNCTION(Heartbeat, event, &JsonRpcConnection::HeartbeatAPIHandler);
 
+/**
+ * We still send a heartbeat without timeout here
+ * to keep the m_Seen variable up to date. This is to keep the
+ * cluster connection alive when there isn't much going on.
+ */
+
 void JsonRpcConnection::HandleAndWriteHeartbeats(boost::asio::yield_context yc)
 {
 	boost::system::error_code ec;
 
 	for (;;) {
-		m_HeartbeatTimer.expires_from_now(boost::posix_time::seconds(10));
+		m_HeartbeatTimer.expires_from_now(boost::posix_time::seconds(20));
 		m_HeartbeatTimer.async_wait(yc[ec]);
 
 		if (m_ShuttingDown) {
 			break;
 		}
 
-		if (m_NextHeartbeat != 0 && m_NextHeartbeat < Utility::GetTime()) {
-			{
-				Log logMsg (LogWarning, "JsonRpcConnection");
-
-				if (m_Endpoint) {
-					logMsg << "Client for endpoint '" << m_Endpoint->GetName() << "'";
-				} else {
-					logMsg << "Anonymous client";
-				}
-
-				logMsg << " has requested heartbeat message but hasn't responded in time. Closing connection.";
-			}
-
-			Disconnect();
-			break;
-		}
-
 		SendMessageInternal(new Dictionary({
 			{ "jsonrpc", "2.0" },
 			{ "method", "event::Heartbeat" },
-			{ "params", new Dictionary({
-				{ "timeout", 120 }
-			}) }
+			{ "params", new Dictionary() }
 		}));
 	}
 }
 
 Value JsonRpcConnection::HeartbeatAPIHandler(const MessageOrigin::Ptr& origin, const Dictionary::Ptr& params)
 {
-	Value vtimeout = params->Get("timeout");
-
-	if (!vtimeout.IsEmpty()) {
-		origin->FromClient->m_NextHeartbeat = Utility::GetTime() + vtimeout;
-	}
-
 	return Empty;
 }
 
